const { dirname } = require("path");
const fs = require("fs");
const { isEmpty } = require("lodash");
const needle = require("needle");
const fileType = require("file-type");
const axios = require("axios");

function toRad(Value) {
    return (Value * Math.PI) / 180;
}

//This function takes in latitude and longitude of two location and returns the distance between them as the crow flies (in km)
function calculateDistanceInKm(lat1, lon1, lat2, lon2) {
    var R = 6371; // km
    var dLat = toRad(lat2 - lat1);
    var dLon = toRad(lon2 - lon1);
    var lat1 = toRad(lat1);
    var lat2 = toRad(lat2);

    var a =
        Math.sin(dLat / 2) * Math.sin(dLat / 2) +
        Math.sin(dLon / 2) *
            Math.sin(dLon / 2) *
            Math.cos(lat1) *
            Math.cos(lat2);
    var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
    var d = R * c;
    return d;
}

function calculateDistanceInMeters(lat1, lon1, lat2, lon2) {
    return calculateDistanceInKm(lat1, lon1, lat2, lon2) * 1000;
}

module.exports = { calculateDistanceInMeters };

async function getUserToken(user_id) {
    return await User.findOne({ where: { id: user_id } });
}

async function saveImage(
    media,
    M_chat_id,
    DataType,
    socket,
    data,
    media_type2,
    user_type,
    user_id,
    room_id,
    user_name,
    receiver
) {
    const url = "https://dashboard.delivery.digisolapps.com/api/save_message_media";

    //u_type => url with type
    const u_type = [];

    if (media) {
        var formData = {
            image: media,
            message_id: M_chat_id,
            media_type: media_type2,
        };
        console.log(formData);
        await needle.post(
            url,
            formData,
            { multipart: true },
            async (err, res) => {
                if (err) {
                    console.error(err);
                }
                for (let i = 0; i < media.length; i++) {
                    let urls = res.body;
                    console.log("$urls" + urls);
                    ext = await fileType.fromBuffer(
                        new Buffer(media[i], "base64")
                    );
                    media_type =
                        ext != null ? await getMediaType(ext.mime) : {};
                    u_type.push([urls[i], media_type]);
                }

                var url = await getUserImage(user_id);
                console.log("$urls" + url);

                $message = {
                    type: DataType,
                    text: !isEmpty(data.message) ? data.message : null,
                    medias: u_type,
                    user_type: user_type,
                    image: url,
                    name: user_name,
                };

                socket.emit("success", {
                    media_type: media_type2[0],
                    image: u_type,
                });
                console.log(room_id);
                socket.to(room_id).emit("response", $message);
                // if (receiver !== null) {
                //     console.log("receiver++++++++++++++++++" + receiver)
                //     socket.broadcast.emit("last_message" + receiver, {
                //         sender:user_id,
                //         type: DataType,
                //         text: !isEmpty(data.message) ? data.message : null,
                //         medias: u_type
                //     });
                // }
            }
        );

        socket.emit("success", { type: DataType, media_type: media_type2[0] });
    } else {
        return u_type;
    }
}

async function getUserImage(id) {
    const url =
        "https://dashboard.delivery.digisolapps.com/api/get-user-image?id=" + id;
    const getData = async (url) => {
        try {
            const response = await axios.get(url);
            const data = response.data;
            return data.data;
        } catch (error) {
            console.log(error);
        }
    };

    return await getData(url);
    // var image="";
    // await  needle
    //     .post(url, {id: id}, {multipart: true}, async (err, res) => {
    //         if (err) {
    //             console.error(err);
    //         }
    //          image = res.body;
    //
    //     });
    // return image;
}

async function getNotification(lang) {
    const response = await fetch(`/Lang/${lang}.json`);
    console.log(response);
}

async function getMediaType(mime) {
    let type = 0;

    switch (mime) {
        case getExt(mime, "image"):
            type = 2;
            break;
        case getExt(mime, "video"):
            type = 3;
            break;
        case getExt(mime, "audio"):
            type = 4;
            break;
        case getExt(mime, "application"):
            type = 5;
            break;
        default:
            type = 6;
    }
    console.log(type);
    return type;
}

function getMainType(data) {
    let type2 = 1;
    if (!isEmpty(data.image) && data.message) {
        type2 = 3;
    } else if (!isEmpty(data.image)) {
        type2 = 2;
    } else if (data.message) {
        type2 = 1;
    }
    return type2;
}

function getExt(text, type) {
    console.log(text.substr(0, text.search("/")) === type);
    if (text.substr(0, text.search("/")) === type) {
        return (
            text.substr(0, text.search("/")) +
            "/" +
            text.substr(text.search("/") + 1)
        );
    }
    return "error";
}

function check_path(path) {
    const appDir = dirname(require.main.filename);
    var dir =
        "/home/deliverydigisola/public_html/delivery/public/uploads/" + path;

    if (!fs.existsSync(dir)) {
        console.log("create new dir ");
        fs.mkdirSync(dir);
    }
}

module.exports = {
    calculateDistanceInKm,
    calculateDistanceInMeters,
    saveImage,
    getMediaType,
    getMainType,
    getUserImage,
};
